import { useEffect, useState } from 'react';
import InputMask from 'react-input-mask';
import { useNavigate, useParams } from 'react-router-dom';
import { ButtonCard } from '../components';
import TextField from '../components/TextField';
import { deleteResponsible, ResponsiblePayload } from '../services/responsible';
import { deleteSpecialist, SpecialistPayload } from '../services/specialist';
import { TeacherPayload, deleteTeacher } from '../services/teacher';
import colors from '../utils/colors';
import fontawesome from '../utils/fontawesome';
import Modal from './Modal';

export interface FormComponentData {
  name: string;
  phone: string;
  email: string;
  cpf: string;
  formation?: string;
  discipline?: string;
}

const defaultProps = {
  className: 'formComponent',
  professor: false,
  specialist: false,
  responsible: false,
  responsibleData: {},
  teacherData: {},
  specData: {},
  isEdicao: false,
  // eslint-disable-next-line react/default-props-match-prop-types
  onClickSubmitTeacher: (teacher: TeacherPayload) => {
    console.log(teacher);
  },
  // eslint-disable-next-line react/default-props-match-prop-types
  onClickSubmitSpec: (spec: SpecialistPayload) => {
    console.log(spec);
  },
  // eslint-disable-next-line react/default-props-match-prop-types
  onClickSubmitResp: (responsible: ResponsiblePayload) => {
    console.log(responsible);
  },
};

type formComponentProps = {
  className?: string;
  professor?: boolean;
  specialist?: boolean;
  responsible?: boolean;
  responsibleData?: ResponsiblePayload;
  teacherData?: TeacherPayload;
  specData?: SpecialistPayload;
  isEdicao?: boolean;
  // eslint-disable-next-line react/require-default-props
  onClickSubmitTeacher: (teacher: TeacherPayload) => void;
  // eslint-disable-next-line react/require-default-props
  onClickSubmitSpec: (specialist: SpecialistPayload) => void;
  // eslint-disable-next-line react/require-default-props
  onClickSubmitResp: (responsible: ResponsiblePayload) => void;
} & typeof defaultProps;

function FormComponent(props: formComponentProps) {
  const navigate = useNavigate();
  const {
    className,
    professor,
    specialist,
    responsible,
    responsibleData,
    teacherData,
    specData,
    isEdicao,
    onClickSubmitTeacher,
    onClickSubmitSpec,
    onClickSubmitResp,
  } = props;

  const styles = {
    container: {
      flex: 1,
      background: colors.theme.background,
      minHeight: '100vh',
      textAlign: 'center' as const,
    },
    modal: {
      position: 'absolute' as const,
      alignself: 'center',
      justifyContent: 'center',
      top: 0,
      left: 0,
      right: 0,
      bottom: 0,
    },
    buttons: {
      display: 'flex',
      justifyContent: 'center',
      marginTop: '5em',
    },
    button: {
      margin: 10,
    },
    wrapper: {
      alignItems: 'center',
      textAlign: 'center' as const,
      display: 'flex',
      justifyContent: 'center',
      marginTop: '5em',
    },
  };

  const [modalEmpty, setModalEmpty] = useState(false);
  const [modalSuccess, setModalSuccess] = useState(false);
  const [modalDelete, setModalDelete] = useState(false);
  const [modalDeleteConfirmed, setModalDeleteConfirmed] = useState(false);

  const toggleModalEmpty = () => {
    setModalEmpty(!modalEmpty);
  };
  const toggleModalSuccess = () => {
    setModalSuccess(!modalSuccess);
  };
  const toggleModalDelete = () => {
    setModalDelete(!modalDelete);
  };
  const toggleModalDeleteConfirmed = () => {
    setModalDeleteConfirmed(!modalDeleteConfirmed);
  };

  const { id } = useParams();
  const handleClickDelete = () => {
    if (professor) deleteTeacher(Number(id));
    if (specialist) deleteSpecialist(Number(id));
    if (responsible) deleteResponsible(Number(id));
  };
  const [name, setName] = useState('');
  const [phone, setPhone] = useState('');
  const [email, setEmail] = useState('');
  const [cpf, setCpf] = useState('');
  const [formation, setFormation] = useState('');
  const [discipline, setDiscipline] = useState('');
  const [nameRed, setNameRed] = useState(false);
  const [cpfRed, setCpfRed] = useState('#4F4F4F');
  const [phoneRed, setPhoneRed] = useState('#4F4F4F');
  const [emailRed, setEmailRed] = useState(false);
  const [specialistRed, setSpecialistRed] = useState(false);
  const validCpf = /^[0-9]{3}\.?[0-9]{3}\.?[0-9]{3}-?[0-9]{2}$/;
  const validPhone = /^\(?[0-9]{2}\)? ?[0-9]{5}-?[0-9]{4}$/;
  const validEmail = /^[a-z0-9.]+@[a-z0-9]+\.[a-z]+\.?([a-z]+)?$/i;

  const createTeacherPayload = (): TeacherPayload => {
    const payloadTeacher: TeacherPayload = {
      name,
      phone_number: phone,
      email,
      document: cpf,
      password: cpf,
    };
    if (discipline.length > 0) {
      payloadTeacher.subject = discipline;
    }
    return payloadTeacher;
  };

  const createSpecialistPayload = (): SpecialistPayload => {
    const payloadSpec: SpecialistPayload = {
      name,
      phone_number: phone,
      email,
      document: cpf,
      formation,
      password: cpf,
    };
    return payloadSpec;
  };

  const createResponsiblePayload = (): ResponsiblePayload => {
    const payloadResp: ResponsiblePayload = {
      name,
      phone_number: phone,
      email,
      document: cpf,
      password: cpf,
    };
    return payloadResp;
  };

  useEffect(() => {
    if (professor) {
      setName(teacherData.name);
      setPhone(teacherData.phone_number);
      setEmail(teacherData.email);
      setCpf(teacherData.document);
      if (teacherData.subject) setDiscipline(teacherData.subject);
    }
    if (specialist) {
      setName(specData.name);
      setPhone(specData.phone_number);
      setEmail(specData.email);
      setCpf(specData.document);
      setFormation(specData.formation);
    }
    if (responsible) {
      setName(responsibleData.name);
      setPhone(responsibleData.phone_number);
      setEmail(responsibleData.email);
      setCpf(responsibleData.document);
    }
  }, [teacherData, specData, responsibleData]);

  const onConfirm = () => {
    if (
      cpf === undefined ||
      !validCpf.test(cpf) ||
      phone === undefined ||
      !validPhone.test(phone) ||
      name === undefined ||
      email === undefined ||
      !validEmail.test(email) ||
      (specialist && formation === undefined)
    ) {
      if (cpf === undefined) {
        setCpfRed(colors.theme.cancel);
      } else if (!validCpf.test(cpf)) {
        setCpfRed(colors.theme.cancel);
      } else {
        setCpfRed('#4F4F4F');
      }
      if (phone === undefined) {
        setPhoneRed(colors.theme.cancel);
      } else if (!validPhone.test(phone)) {
        setPhoneRed(colors.theme.cancel);
      } else {
        setPhoneRed('#4F4F4F');
      }
      if (name === undefined) {
        setNameRed(true);
      } else if (name.length === 0) {
        setNameRed(true);
      } else {
        setNameRed(false);
      }
      if (email === undefined) {
        setEmailRed(true);
      } else if (email.length === 0) {
        setEmailRed(true);
      } else if (!validEmail.test(email)) {
        setEmailRed(true);
      } else {
        setEmailRed(false);
      }
      if (specialist && formation === undefined) {
        setSpecialistRed(true);
      } else if (specialist && formation.length === 0) {
        setSpecialistRed(true);
      } else {
        setSpecialistRed(false);
      }

      toggleModalEmpty();
    } else if (
      name.length === 0 ||
      email.length === 0 ||
      !validEmail.test(email) ||
      (specialist && formation.length === 0)
    ) {
      if (name.length === 0) {
        setNameRed(true);
      } else {
        setNameRed(false);
      }
      if (email.length === 0) {
        setEmailRed(true);
      } else if (!validEmail.test(email)) {
        setEmailRed(true);
      } else {
        setEmailRed(false);
      }
      if (specialist && formation.length === 0) {
        setSpecialistRed(true);
      } else {
        setSpecialistRed(false);
      }

      toggleModalEmpty();
    } else {
      setNameRed(false);
      setCpfRed('#4F4F4F');
      setPhoneRed('#4F4F4F');
      setEmailRed(false);
      setSpecialistRed(false);
      toggleModalSuccess();
      if (professor) onClickSubmitTeacher(createTeacherPayload());
      if (specialist) onClickSubmitSpec(createSpecialistPayload());
      if (responsible) onClickSubmitResp(createResponsiblePayload());
    }
  };

  const renderButtons = () => {
    return !isEdicao ? (
      <div
        style={{
          display: 'flex',
          justifyContent: 'center',
          padding: '1.5625rem',
        }}
      >
        <ButtonCard
          width="16.28em"
          height="3.29em"
          text=""
          icon={fontawesome.icons.faCheck}
          sizeIcon="2.25em"
          marginIcon="0rem"
          backgroundColor={colors.theme.blue}
          iconWidth="100%"
          onClick={onConfirm}
        />
      </div>
    ) : (
      <div style={{ display: 'flex', justifyContent: 'center' }}>
        <div style={{ margin: '1em' }}>
          <ButtonCard
            width="8.28em"
            height="3.29em"
            text=""
            icon={fontawesome.icons.faTrash}
            sizeIcon="2.25em"
            marginIcon="0em"
            backgroundColor={colors.theme.cancel}
            iconWidth="100%"
            onClick={() => {
              toggleModalDelete();
            }}
          />
        </div>
        <div style={{ margin: '1em' }}>
          <ButtonCard
            width={isEdicao ? '8.28em' : '16.28em'}
            height="3.29em"
            text=""
            icon={fontawesome.icons.faCheck}
            sizeIcon="2.25em"
            marginIcon="0rem"
            backgroundColor={colors.theme.blue}
            iconWidth="100%"
            onClick={onConfirm}
          />
        </div>
      </div>
    );
  };

  return (
    <div className={className} style={{ padding: '20px' }}>
      <div style={{ display: 'flex', justifyContent: 'center' }}>
        <TextField
          placeholder="Nome Completo"
          borderColor={nameRed ? colors.theme.cancel : colors.theme.text_icon}
          acceptValue
          value={name}
          onChange={event => setName(event.target.value)}
        />
      </div>
      <div style={{ display: 'flex', justifyContent: 'center' }}>
        <InputMask
          style={{
            width: '85%',
            maxWidth: '66.8rem',
            minWidth: '20.1875rem',
            height: '3.25rem',
            margin: '1.5625rem 0rem 0rem 0rem',
            padding: '17px 14px 17px 17px',
            borderRadius: '0.3125rem',
            border: '1px solid',
            borderColor: `${phoneRed}`,
          }}
          placeholder="Telefone"
          mask="(99) 99999-9999"
          value={phone}
          onChange={(event: any) => {
            setPhone(event.target.value);
          }}
        />
      </div>

      <div style={{ display: 'flex', justifyContent: 'center' }}>
        <TextField
          placeholder="E-mail"
          borderColor={emailRed ? colors.theme.cancel : colors.theme.text_icon}
          acceptValue
          value={email}
          onChange={event => setEmail(event.target.value)}
        />
      </div>
      <div style={{ display: 'flex', justifyContent: 'center' }}>
        <InputMask
          style={{
            width: '85%',
            maxWidth: '66.8rem',
            minWidth: '20.1875rem',
            height: '3.25rem',
            margin: '1.5625rem 0rem 0rem 0rem',
            padding: '17px 14px 17px 17px',
            borderRadius: '0.3125rem',
            border: '1px solid',
            borderColor: `${cpfRed}`,
          }}
          placeholder="CPF"
          mask="999.999.999-99"
          value={cpf}
          onChange={(event: any) => {
            setCpf(event.target.value);
          }}
        />
      </div>

      {specialist && (
        <div style={{ display: 'flex', justifyContent: 'center' }}>
          <TextField
            placeholder="Especialização"
            borderColor={
              specialistRed ? colors.theme.cancel : colors.theme.text_icon
            }
            acceptValue
            value={formation}
            onChange={event => setFormation(event.target.value)}
          />
        </div>
      )}
      {professor && (
        <div style={{ display: 'flex', justifyContent: 'center' }}>
          <TextField
            placeholder="Disciplina (Opcional)"
            acceptValue
            value={discipline}
            onChange={event => setDiscipline(event.target.value)}
          />
        </div>
      )}

      {renderButtons()}

      {modalEmpty && (
        <div style={styles.modal}>
          <Modal
            text="Preencha todos os campos necessários corretamente."
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={toggleModalEmpty}
          />
        </div>
      )}
      {modalSuccess && (
        <div style={styles.modal}>
          <Modal
            text="Usuário cadastrado com successo."
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={() => {
              toggleModalSuccess();
              navigate(-1);
            }}
          />
        </div>
      )}
      {modalDelete && (
        <div style={styles.modal}>
          <Modal
            text="Tem certeza que deseja excluir este usuário?"
            closeModal={toggleModalDelete}
            noButton={toggleModalDelete}
            checkButton={() => {
              toggleModalDelete();
              handleClickDelete();
              toggleModalDeleteConfirmed();
            }}
          />
        </div>
      )}
      {modalDeleteConfirmed && (
        <div style={styles.modal}>
          <Modal
            text="Usuário deletado com sucesso."
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={() => {
              toggleModalDeleteConfirmed();
              navigate(-1);
            }}
          />
        </div>
      )}
    </div>
  );
}

FormComponent.defaultProps = defaultProps;

export default FormComponent;
