import { useEffect, useState } from 'react';
import { useNavigate, useParams } from 'react-router-dom';
import { ButtonCard, TextField, TitleComponent } from '../components';
import TextArea from '../components/TextArea';
import { Header, Modal } from '../containers';
import { StudentData } from '../model/student';
import {
  createStudentData,
  CreateStudentDataPayload,
  deleteStudentData,
  editStudentData,
  getStudentDataById,
} from '../services/studentData';
import colors from '../utils/colors';
import fontawesome from '../utils/fontawesome';

const styles = {
  modal: {
    position: 'absolute' as const,
    alignself: 'center',
    justifyContent: 'center',
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
  },
};

const defaultProps = {
  justView: false,
};

type adicionarDadoProps = {
  justView?: boolean;
} & typeof defaultProps;

function AdicionarDado({ justView }: adicionarDadoProps) {
  const { idAluno, id } = useParams();
  const navigate = useNavigate();
  const [isEdicao, setIsEdicao] = useState<boolean>(false);
  const [subject, setSubject] = useState<string>('');
  const [content, setContent] = useState<string>('');
  const [modalDelete, setModalDelete] = useState(false);
  const [modalSuccess, setModalSuccess] = useState(false);
  const [modalEmpty, setModalEmpty] = useState(false);
  const [modalDeleteConfirmed, setModalDeleteConfirmed] = useState(false);

  const toggleModalSuccess = () => {
    setModalSuccess(!modalSuccess);
  };
  const toggleModalDelete = () => {
    setModalDelete(!modalDelete);
  };
  const toggleModalEmpty = () => {
    setModalEmpty(!modalEmpty);
  };
  const toggleModalDeleteConfirmed = () => {
    setModalDeleteConfirmed(!modalDeleteConfirmed);
  };

  const handleClickSave = (data: CreateStudentDataPayload) => {
    if (isEdicao) editStudentData(Number(idAluno), data, Number(id));
    else createStudentData(Number(idAluno), data);
  };
  const handleClickDelete = () =>
    deleteStudentData(Number(idAluno), Number(id));

  useEffect(() => {
    const getData = async (studentId: number, id: number) => {
      try {
        const getStudentDataResponse: StudentData = await getStudentDataById(
          studentId,
          id,
        );
        setSubject(getStudentDataResponse.subject);
        setContent(getStudentDataResponse.content);
      } catch (err) {
        console.log(err);
      }
    };
    if (id) {
      setIsEdicao(true);
      getData(Number(idAluno), Number(id));
    }
  }, []);

  return (
    <div
      style={{
        backgroundColor: colors.theme.background,
        minHeight: '100vh',
        paddingBottom: '2rem',
        marginTop: '6.25rem',
      }}
    >
      <div className="fixed-top">
        <Header text="Área do Aluno" disableArrow enableLogout />
      </div>

      <div style={{ paddingTop: '1.563rem' }}>
        <TitleComponent
          text={
            // eslint-disable-next-line no-nested-ternary
            justView
              ? 'Visualizar Dado'
              : isEdicao
              ? 'Editar Dado'
              : 'Adicionar Dado'
          }
          textSize="1.75em"
          fontWeight="600"
        />
      </div>
      <div style={{ display: 'flex', justifyContent: 'center' }}>
        <TextField
          placeholder="Digite um aspecto do aluno"
          acceptValue
          value={subject}
          onChange={event => setSubject(event.target.value)}
          enableReadonly={justView}
        />
      </div>
      <div
        style={{
          marginTop: '1.563rem',
        }}
      >
        <TextArea
          placeholder="Digite o detalhamento de tal aspecto"
          height="25.5625rem"
          acceptValue
          value={content}
          setChange={event => setContent(event.target.value)}
          enableReadonly={justView}
        />
      </div>
      {isEdicao
        ? !justView && (
            <div
              style={{
                display: 'flex',
                justifyContent: 'center',
                marginTop: '2.75rem',
              }}
            >
              <div style={{ margin: '1em' }}>
                <ButtonCard
                  text=""
                  backgroundColor={colors.theme.cancel}
                  width="8.28em"
                  height="3.29em"
                  iconWidth="100%"
                  sizeIcon="2.25em"
                  marginIcon="0rem"
                  icon={fontawesome.icons.faTrash}
                  onClick={toggleModalDelete}
                />
              </div>
              <div style={{ margin: '1em' }}>
                <ButtonCard
                  text=""
                  backgroundColor={colors.theme.blue}
                  width="8.28em"
                  height="3.29em"
                  iconWidth="100%"
                  sizeIcon="2.25em"
                  marginIcon="0rem"
                  icon={fontawesome.icons.faCheck}
                  onClick={() => {
                    handleClickSave({ subject, content });
                    toggleModalSuccess();
                  }}
                />
              </div>
            </div>
          )
        : !justView && (
            <div
              style={{
                display: 'flex',
                justifyContent: 'center',
                marginTop: '2.75rem',
              }}
            >
              <ButtonCard
                text=""
                backgroundColor={colors.theme.blue}
                width="16.28em"
                height="3.29em"
                marginIcon="0rem"
                iconWidth="100%"
                sizeIcon="2.25em"
                icon={fontawesome.icons.faCheck}
                onClick={() => {
                  if (subject.length === 0 || content.length === 0) {
                    toggleModalEmpty();
                  } else {
                    handleClickSave({ subject, content });
                    toggleModalSuccess();
                  }
                }}
              />
            </div>
          )}
      {modalSuccess && (
        <div style={styles.modal}>
          <Modal
            text="Dado cadastrado com sucesso."
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={() => navigate(-1)}
          />
        </div>
      )}
      {modalDelete && (
        <div style={styles.modal}>
          <Modal
            text="Tem certeza que deseja excluir este dado?"
            closeModal={toggleModalDelete}
            checkButton={() => {
              handleClickDelete();
              navigate(-1);
            }}
            noButton={toggleModalDelete}
          />
        </div>
      )}
      {modalEmpty && (
        <div style={styles.modal}>
          <Modal
            text="Preencha todos os campos necessários."
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={toggleModalEmpty}
          />
        </div>
      )}
      {modalDeleteConfirmed && (
        <div style={styles.modal}>
          <Modal
            text="Dado deletado com sucesso."
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={() => {
              toggleModalDeleteConfirmed();
              navigate(-1);
            }}
          />
        </div>
      )}
    </div>
  );
}

AdicionarDado.defaultProps = defaultProps;

export default AdicionarDado;
