import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { DropDown, TextField, TitleComponent } from '../components';
import { CardLista, Header } from '../containers';
import { Category } from '../model/category';
import { User } from '../model/user';
import { getAllCategory } from '../services/category';
import { getAllPosts, Post } from '../services/post';
import colors from '../utils/colors';
import fontawesome from '../utils/fontawesome';
import { FormatDate } from '../utils/formatDate';

function ForumList() {
  const navigate = useNavigate();
  const [tags, setTags] = useState<Category[]>([]);
  const [tagsSearch, setTagsSearch] = useState(Array<string>());
  const [searchPostName, setSearchPostName] = useState<string>('');
  const [posts, setPosts] = useState<Post[]>([]);
  const [filteredPosts, setFilteredPosts] = useState<Post[]>([]);

  const userFromSession: User = JSON.parse(
    sessionStorage.getItem('user') || '{}',
  );

  const postArrayHandlerByTitle = (
    event: React.ChangeEvent<HTMLInputElement>,
  ) => {
    const searchedPostName: string = event.target.value;
    const newFilteredPostsTitle: Post[] = posts.filter(auxPost =>
      auxPost.title.toLowerCase().includes(searchedPostName.toLowerCase()),
    );
    let newFilteredPostsTag: Post[];
    if (tagsSearch.length === 0) {
      newFilteredPostsTag = newFilteredPostsTitle;
    } else {
      newFilteredPostsTag = newFilteredPostsTitle.filter(auxPost =>
        tagsSearch.every(selectedTag => auxPost.tags.includes(selectedTag)),
      );
    }
    setSearchPostName(searchedPostName);
    setFilteredPosts(newFilteredPostsTag);
  };

  const postArrayHandlerByTag = (selectedTags: string[]) => {
    const newFilteredPostsTitle: Post[] = posts.filter(auxPost =>
      auxPost.title.toLowerCase().includes(searchPostName.toLowerCase()),
    );

    let newFilteredPostsTag: Post[];
    if (selectedTags.length === 0) {
      newFilteredPostsTag = newFilteredPostsTitle;
    } else {
      newFilteredPostsTag = newFilteredPostsTitle.filter(auxPost =>
        selectedTags.every(selectedTag => auxPost.tags.includes(selectedTag)),
      );
    }
    setTagsSearch(selectedTags);
    setFilteredPosts(newFilteredPostsTag);
  };

  useEffect(() => {
    const getCategories = async () => {
      try {
        const getCategoriesResponse: Category[] = await getAllCategory();
        setTags(getCategoriesResponse);
      } catch (err) {
        console.log(err);
      }
    };
    const getPosts = async () => {
      try {
        const getPostResponse: Post[] = await getAllPosts();
        setPosts(
          getPostResponse.filter(post => {
            if (userFromSession.type === 'SPECIALIST') {
              if (post.visible_specialist) {
                return true;
              }
              return false;
            }
            return true;
          }),
        );
        setFilteredPosts(
          getPostResponse.filter(post => {
            if (userFromSession.type === 'SPECIALIST') {
              if (post.visible_specialist) {
                return true;
              }
              return false;
            }
            return true;
          }),
        );
      } catch (err) {
        console.log(err);
      }
    };
    getPosts();
    getCategories();
  }, []);

  const styles = {
    titleContainer: {
      display: 'flex',
      justifyContent: 'center',
      paddingTop: '9.5rem',
      paddingBottom: '1.375rem',
    },
    inputContainer: {
      display: 'flex',
      justifyContent: 'center',
    },
    wrap: {
      textAlign: 'center' as const,
      paddingBottom: '10px',
    },
    wrap2: {
      display: 'flex',
      justifyContent: 'center',
    },
    background: {
      backgroundColor: colors.theme.background,
      minHeight: '100vh',
    },
    notMaterialText: {
      textAlign: 'center' as const,
      paddingTop: '20px',
      fontStyle: 'poppins',
    },
    icon: {
      fontSize: '2rem',
      marginLeft: '1.75rem',
      cursor: 'pointer',
      paddingTop: '0.3rem',
    },
    card: {
      marginBottom: '1.438rem',
      minWidth: '20.188rem',
      maxWidth: '59rem',
      width: '85%',
    },
    postList: {
      display: 'flex',
      flexDirection: 'column' as const,
      alignItems: 'center',
      padding: '2.438rem 0rem',
    },
  };

  return (
    <div style={styles.background}>
      <div className="fixed-top">
        <Header
          text={
            userFromSession.type === 'TEACHER'
              ? 'Área do Professor'
              : 'Área do Especialista'
          }
          disableArrow
          enableLogout
        />
      </div>
      <div style={styles.titleContainer}>
        <TitleComponent text="Fórum" fontWeight="600" textSize="1.75rem" />
        <FontAwesomeIcon
          icon={fontawesome.icons.faSquarePlus}
          style={styles.icon}
          onClick={() => navigate('/adicionar-postagem')}
        />
      </div>
      <div>
        <div style={styles.inputContainer}>
          <TextField
            placeholder="Buscar..."
            borderRadius="5px"
            width="20.1875rem"
            paddingText="1.0625rem 0.875rem 1.0625rem 0.72rem"
            margin="0px 0px 14px 0px"
            disableIcon={false}
            onChange={postArrayHandlerByTitle}
          />
        </div>
        <div style={styles.inputContainer}>
          <DropDown
            selectText="Buscar tags..."
            widthContainer="20.1875rem"
            options={tags.map(tag => {
              return {
                value: tag.name,
                label: tag.name.charAt(0) + tag.name.substring(1).toLowerCase(),
              };
            })}
            val={tagsSearch}
            onChange={value => {
              const selected = Array<string>();
              for (let i = 0; i < value.length; i++) {
                selected.push(value[i].value);
              }
              postArrayHandlerByTag(selected);
            }}
          />
        </div>
        <div style={styles.postList}>
          {filteredPosts.length !== 0 ? (
            filteredPosts.map(post => {
              return (
                <div key={post.id} style={styles.card}>
                  <CardLista
                    titleContent={`${post.title} - ${FormatDate(post.date)}`}
                    subtitle1Content={`Postado por: ${post.author.name} (${
                      post.author.type === 'TEACHER'
                        ? 'professor'
                        : 'especialista'
                    })`}
                    fontSizeTitle="1.20rem"
                    fontSizeSubtitle="0.875rem"
                    enableSubtitle2={false}
                    enableIconArrow={false}
                    enableIconPen={false}
                    tagContent={post.tags}
                    marginSubtitle="0rem 0rem 0.5rem 0rem"
                    enableCardClick
                    enableHR
                    marginCard="0rem"
                    // COLOCAR PARA IR PARA A ROTA CERTA
                    // onClickCard={() => {
                    //   navigate(`/forum-lista-posts/${post.id}`);
                    // }}
                  />
                </div>
              );
            })
          ) : (
            <div style={styles.card}>
              <CardLista
                titleContent={
                  posts.length !== 0
                    ? 'Essa postagem nao foi encontrada'
                    : `Nenhuma postagem foi cadastrada`
                }
                enableIconArrow={false}
                enableIconPen={false}
                enableSubtitle1={false}
                enableSubtitle2={false}
                enableTag={false}
                disableTitleCenter={false}
                textColor={colors.theme.cancel}
                marginCard="0rem"
                onlyTitle
              />
            </div>
          )}
        </div>
      </div>
    </div>
  );
}

export default ForumList;
