import { useEffect, useState } from 'react';
import { useNavigate, useParams } from 'react-router-dom';
import { ButtonCard, TitleComponent } from '../components';
import TextArea from '../components/TextArea';
import { Header, Modal } from '../containers';
import { Report } from '../model/report';
import {
  getReportById,
  ReportUpdatePayload,
  updateReport,
  send,
} from '../services/report';
import colors from '../utils/colors';
import fontawesome from '../utils/fontawesome';
import { FormatDate } from '../utils/formatDate';

const styles = {
  modal: {
    position: 'absolute' as const,
    alignself: 'center',
    justifyContent: 'center',
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
  },
};

const defaultProps = {
  justView: false,
};

type reportViewEditProps = {
  justView?: boolean;
} & typeof defaultProps;

function ReportViewEditProps({ justView }: reportViewEditProps) {
  const navigate = useNavigate();
  const { idAluno, id } = useParams();
  const [content, setContent] = useState<string>('');
  const [reportData, setReportData] = useState<Report>();
  const [dateAuxStart, setDateAuxStart] = useState<string>('');
  const [dateAuxEnd, setDateAuxEnd] = useState<string>('');

  const [modalSuccess, setModalSuccess] = useState(false);
  const [modalEmpty, setModalEmpty] = useState(false);
  const [modalEmail, setModalEmail] = useState(false);

  const toggleModalSuccess = () => {
    setModalSuccess(!modalSuccess);
  };
  const toggleModalEmpty = () => {
    setModalEmpty(!modalEmpty);
  };
  const toggleModalEmail = () => {
    setModalEmail(!modalEmail);
  };

  const handleClickSave = (data: ReportUpdatePayload) => {
    updateReport(Number(id), data);
  };

  const sendEmail = () => {
    send(Number(id));
  };

  useEffect(() => {
    const getReport = async (id: number) => {
      try {
        const getReportDataResponse: Report = await getReportById(id);
        setReportData(getReportDataResponse);
        setContent(getReportDataResponse.content);
        setDateAuxStart(getReportDataResponse.date_start);
        setDateAuxEnd(getReportDataResponse.date_delivery);
      } catch (err) {
        console.log(err);
      }
    };
    getReport(Number(id));
  }, []);

  return (
    <div
      style={{
        backgroundColor: colors.theme.background,
        minHeight: '100vh',
        paddingBottom: '2rem',
        marginTop: '6.25rem',
      }}
    >
      <div className="fixed-top">
        <Header text="Área do Aluno" disableArrow enableLogout />
      </div>

      <div style={{ paddingTop: '1.563rem' }}>
        <TitleComponent
          text={justView ? 'Visualizar Relatório' : 'Editar Relatório'}
          textSize="1.75em"
          fontWeight="600"
        />
      </div>
      <div>
        {justView && (
          <TitleComponent
            text={`Professor: ${reportData?.teacher.name}`}
            textSize="1.125em"
            fontWeight="600"
          />
        )}
        <TitleComponent
          text={`Período: ${FormatDate(dateAuxStart)}
           - ${FormatDate(dateAuxEnd)}`}
          textSize="1.125em"
          fontWeight="600"
        />
      </div>
      <div
        style={{
          marginTop: '1.563rem',
          marginBottom: '2.75rem',
        }}
      >
        <TextArea
          placeholder="Escreva o que deseja colocar no relatório"
          height="25.5625rem"
          acceptValue
          value={content}
          setChange={event => setContent(event.target.value)}
        />
      </div>
      <div
        style={{
          display: 'flex',
          justifyContent: 'center',
        }}
      >
        <ButtonCard
          text=""
          backgroundColor={colors.theme.blue}
          width="8.75rem"
          height="3.25rem"
          iconWidth="100%"
          sizeIcon="2.25em"
          marginIcon="0rem"
          icon={fontawesome.icons.faCheck}
          onClick={() => {
            if (content.length === 0) {
              toggleModalEmpty();
            } else {
              handleClickSave({
                content,
                date_start: FormatDate(dateAuxStart),
                date_delivery: FormatDate(dateAuxEnd),
              });
              toggleModalSuccess();
            }
          }}
        />

        <ButtonCard
          text=""
          backgroundColor={colors.theme.blue}
          width="8.75rem"
          height="3.25rem"
          iconWidth="100%"
          sizeIcon="2.25em"
          marginIcon="0rem"
          margin="0rem 0rem 0rem 2.563rem"
          icon={fontawesome.icons.faPaperPlane}
          onClick={() => {
            if (content.length === 0) {
              toggleModalEmpty();
            } else {
              sendEmail();
              toggleModalEmail();
            }
          }}
        />
      </div>
      {modalSuccess && (
        <div style={styles.modal}>
          <Modal
            text="Relatório atualizado com sucesso."
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={() => navigate(-1)}
          />
        </div>
      )}
      {modalEmpty && (
        <div style={styles.modal}>
          <Modal
            text="Preencha o campo do conteúdo do relatório"
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={toggleModalEmpty}
          />
        </div>
      )}
      {modalEmail && (
        <div style={styles.modal}>
          <Modal
            text="E-mail enviado com sucesso para o(s) responsável(is)"
            hasWarningSign={false}
            areYouSureModal={false}
            width="17.5em"
            height="9.375em"
            closeModal={() => navigate(-1)}
          />
        </div>
      )}
    </div>
  );
}

ReportViewEditProps.defaultProps = defaultProps;

export default ReportViewEditProps;
