import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { TextField, TitleComponent } from '../components';
import CardLista from '../containers/CardLista';
import Header from '../containers/Header';
import { Student } from '../model/student';
import { User, UserType } from '../model/user';
import { getAllStudents, getAllStudentsByTeacher } from '../services/student';
import colors from '../utils/colors';
import fontawesome from '../utils/fontawesome';
import { getAge } from '../utils/formatDate';

const defaultProps = {
  isCoordinator: false,
};

type studentListScreenProps = {
  isCoordinator?: boolean;
} & typeof defaultProps;

function StudentList({ isCoordinator }: studentListScreenProps) {
  const navigate = useNavigate();
  const [searchName, setSearchName] = useState<string>('');
  const [students, setStudents] = useState<Student[]>([]);
  const [filteredStudents, setFilteredStudents] = useState<Student[]>([]);

  useEffect(() => {
    const getStudents = async () => {
      try {
        const currentSession = sessionStorage.getItem('user');
        let getStudentsResponse: Student[];
        const userSession: User = JSON.parse(currentSession || '{}');
        if (userSession.type === UserType.TEACHER) {
          getStudentsResponse = await getAllStudentsByTeacher(userSession.id);
        } else {
          getStudentsResponse = await getAllStudents();
        }
        setStudents(getStudentsResponse);
        setFilteredStudents(getStudentsResponse);
      } catch (err) {
        console.log(err);
      }
    };
    getStudents();
  }, []);

  const userArrayHandler = (event: React.ChangeEvent<HTMLInputElement>) => {
    const searchedStudentName: string = event.target.value;
    const newFilteredStudents: Student[] = students.filter(testUser =>
      testUser.name.toLowerCase().includes(searchedStudentName.toLowerCase()),
    );
    setSearchName(searchedStudentName);
    setFilteredStudents(newFilteredStudents);
  };

  return (
    <div style={styles.background}>
      <div className="fixed-top">
        <Header
          text={isCoordinator ? 'Área do Coordenador' : 'Lista de Alunos'}
          disableArrow
          enableLogout
        />
      </div>
      {isCoordinator && (
        <>
          <div style={styles.titleContainer}>
            <TitleComponent
              text="Buscar Aluno"
              fontWeight="600"
              textSize="1.75rem"
            />
            <FontAwesomeIcon
              icon={fontawesome.icons.faSquarePlus}
              style={styles.icon}
              onClick={() => navigate('/cadastrar-aluno')}
            />
          </div>
          <div style={styles.searchBar}>
            <TextField
              placeholder="Buscar..."
              margin="0px"
              borderRadius="30px"
              disableIcon={false}
              onChange={userArrayHandler}
            />
          </div>
        </>
      )}
      <div
        style={isCoordinator ? styles.studentListCoord : styles.studentListProf}
      >
        {filteredStudents.length !== 0 ? (
          filteredStudents.map(student => {
            return (
              <div key={student.id} style={styles.card}>
                <CardLista
                  titleContent={student.name}
                  subtitle1Content={student.serie}
                  enableIconPen={isCoordinator}
                  enableIconArrow={!isCoordinator}
                  enableSubtitle2={false}
                  enableTag={false}
                  marginCard="0rem"
                  onClickPen={() => navigate(`/editar-aluno/${student.id}`)}
                  onClickArrow={() => navigate(`/area-aluno/${student.id}`)}
                />
              </div>
            );
          })
        ) : (
          <div style={styles.card}>
            <CardLista
              titleContent={
                students.length !== 0
                  ? `O aluno ${
                      searchName.length >= 12
                        ? `${searchName.slice(0, 11)}...`
                        : searchName
                    } não foi encontrado`
                  : `Nenhum aluno foi cadastrado`
              }
              enableIconArrow={false}
              enableIconPen={false}
              enableSubtitle1={false}
              enableSubtitle2={false}
              enableTag={false}
              disableTitleCenter={false}
              textColor={colors.theme.cancel}
              marginCard="0rem"
              onlyTitle
            />
          </div>
        )}
      </div>
    </div>
  );
}

const styles = {
  background: {
    background: colors.theme.background,
    minHeight: '145vh',
  },
  studentListProf: {
    display: 'flex',
    flexDirection: 'column' as const,
    alignItems: 'center',
    padding: '2.438rem 0rem',
    justifyContent: 'center',
    paddingTop: '9.5rem',
    paddingBottom: '1.375rem',
  },
  studentListCoord: {
    display: 'flex',
    flexDirection: 'column' as const,
    alignItems: 'center',
    padding: '2.438rem 0rem',
  },
  card: {
    marginBottom: '1.438rem',
    minWidth: '20.188rem',
    maxWidth: '59rem',
    width: '85%',
  },
  titleContainer: {
    display: 'flex',
    justifyContent: 'center',
    paddingTop: '9.5rem',
    paddingBottom: '1.375rem',
  },
  searchBar: {
    display: 'flex',
    justifyContent: 'center',
  },
  icon: {
    fontSize: '2rem',
    marginLeft: '1.75rem',
    cursor: 'pointer',
    paddingTop: '0.3rem',
  },
};

StudentList.defaultProps = defaultProps;

export default StudentList;
